package mathvis.ws06.jtem.ode;


public class HeightFieldUtils {

	private HeightFieldUtils() {}
	
	/**
	 * Extracts the inner points of a rectangular height field with hf[i,j]=values[j*u+i].
	 * 
	 * @param values the values of the height field
	 * @param u the extent u-direction
	 * @param v the extent in v-direction
	 * @return the inner points
	 */
	public static double[] getInnerValues(double[] values, int u, int v) {
		double[] ret = new double[(u-2)*(v-2)];
		getInnerValues(values, ret, 0, u, v);
		return ret;
	}
	
	/**
	 * Extracts the inner points of a height field with hf[i,j]=values[j*u+i] and
	 * writes them into a target array at a given offset.
	 * 
	 * @param src the values of the height field
	 * @param u the extent u-direction
	 * @param v the extent in v-direction
	 * @param target the target array
	 * @param targetOffset the offset to write to the target array
	 */
	public static void getInnerValues(double[] src, double[] target, int targetOffset, int u, int v) {
		for (int j = 0; j < v-2; j++) {
			for (int i = 0; i < u-2; i++) {
				target[targetOffset+j*(u-2)+i]=src[(j+1)*u+i+1];
			}
		}
	}

	/**
	 * 
	 * Assigns the values from src starting at srcOffset as 
	 * inner points of the height field (target, u, v) 
	 * 
	 * @param target the values of the height field
	 * @param src the values to write as inner values
	 * @param srcOffset the src offset
	 * @param u the extent u-direction
	 * @param v the extent in v-direction
	 */
	public static void setInnerValues(double[] target, double[] src, int srcOffset, int u, int v) {
		for (int j = 0; j < v-2; j++) {
			for (int i = 0; i < u-2; i++) {
				target[(j+1)*u+i+1]=src[srcOffset+j*(u-2)+i];
			}
		}
	}
	
	/**
	 * calculates the discrete laplacian at (i, j) of the given height field (values, u, v).
	 * @param values the values of the height field
	 * @param u the extent u-direction
	 * @param v the extent in v-direction
	 * @param i the point where to calculate the laplacian
	 * @param j the point where to calculate the laplacian
	 * @return the laplacian at (i, j)
	 */
	public static double laplace(double[] values, int u, int v, int i, int j) {
		if (i<1||j<1||i>u-1||j>v-1) throw new IllegalArgumentException("only for inner points! ["+i+","+j+"]");
		double f = -4*values[j*u+i];
		f += values[j*u+i-1];
		f += values[j*u+i+1];
		f += values[(j-1)*u+i];
		f += values[(j+1)*u+i];
		return f;
	}
	
}
